@@ -1,32 +1,17 @@
-# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Michal Minar <miminar@redhat.com>
 #
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 #
-# 1. Redistributions of source code must retain the above copyright notice,
-#    this list of conditions and the following disclaimer.
-# 2. Redistributions in binary form must reproduce the above copyright notice,
-#    this list of conditions and the following disclaimer in the documentation
-#    and/or other materials provided with the distribution.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
-# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-#
-# The views and conclusions contained in the software and documentation are
-# those of the authors and should not be interpreted as representing official
-# policies, either expressed or implied, of the FreeBSD Project.
-#
-# Authors: Michal Minar <miminar@redhat.com>
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 """
 Module for session object representing all connection to remote hosts.
 """
@@ -35,6 +20,7 @@ from collections import defaultdict
 
 from lmi.scripts.common import errors
 from lmi.scripts.common import get_logger
+from lmi.scripts.common.util import FilteredDict
 from lmi.shell.LMIConnection import connect
 
 LOG = get_logger(__name__)
@@ -57,7 +43,7 @@ class Session(object):
 
     def __init__(self, app, hosts, credentials=None, same_credentials=False):
         self._app = app
-        self._connections = {h: None for h in hosts}
+        self._connections = dict((h, None) for h in hosts)
         # { hostname : (username, password, verified), ... }
         # where verified is a flag saying, whether these credentials
         # were successfuly used for logging in
@@ -76,26 +62,29 @@ class Session(object):
             ``None`` if connection can not be made.
         """
         if self._connections[hostname] is None:
-            self._connections[hostname] = self._connect(
-                    hostname, interactive=True)
+            try:
+                self._connections[hostname] = self._connect(
+                        hostname, interactive=True)
+            except Exception as exc:
+                LOG().error('Failed to make a connection to "%s": %s',
+                        hostname, exc)
         return self._connections[hostname]
 
     def __len__(self):
         """ Get the number of hostnames in session. """
         return len(self._connections)
 
+    def __contains__(self, uri):
+        return uri in self._connections
+
     def __iter__(self):
         """ Yields connection objects. """
         successful_connections = 0
         for hostname in self._connections:
-            try:
-                connection = self[hostname]
-                if connection is not None:
-                    yield connection
-                    successful_connections += 1
-            except Exception as exc:
-                LOG().error('Failed to make a connection to "%s": %s',
-                        hostname, exc)
+            connection = self[hostname]
+            if connection is not None:
+                yield connection
+                successful_connections += 1
         if successful_connections == 0:
             raise errors.LmiNoConnections('No successful connection made.')
 
@@ -160,3 +149,27 @@ class Session(object):
         """
         return [h for h, c in self._connections.items() if c is None]
 
+class SessionProxy(Session):
+    """
+    Behaves like a session. But it just encapsulates other session object and
+    provides access to a subset of its items.
+
+    :param session: Session object or even another session proxy.
+    :param list uris: Subset of uris in encapsulated session object.
+    """
+
+    def __init__(self, session, uris):
+        uris = set(uris)
+        if not all(isinstance(uri, basestring) for uri in uris):
+            raise ValueError("uris must be iterable of uris")
+        for uri in uris:
+            if not uri in session:
+                raise ValueError('uri "%s" needs to belong to given session'
+                        % uri)
+        Session.__init__(self, session._app, uris, session._credentials,
+                session._same_credentials)
+        self._origin = session
+        self._connections = FilteredDict(uris, session._connections)
+        # let the credentials propagage to original session
+        self._credentials = session._credentials
+
